@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -92,16 +92,13 @@ public class XCSoar extends Activity {
     NetUtil.initialise(this);
     InternalGPS.Initialize();
     NonGPSSensors.Initialize();
+    GliderLinkReceiver.Initialize();
 
     IOIOHelper.onCreateContext(this);
 
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ECLAIR)
-      // Bluetooth suppoert was added in Android 2.0 "Eclair"
-      BluetoothHelper.Initialize(this);
+    BluetoothHelper.Initialize(this);
 
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD)
-      // the DownloadManager was added in Android 2.3 "Gingerbread"
-      DownloadUtil.Initialise(this);
+    DownloadUtil.Initialise(this);
 
     // fullscreen mode
     requestWindowFeature(Window.FEATURE_NO_TITLE);
@@ -111,12 +108,10 @@ public class XCSoar extends Activity {
     /* Workaround for layout problems in Android KitKat with immersive full
        screen mode: Sometimes the content view was not initialized with the
        correct size, which caused graphics artifacts. */
-    if (android.os.Build.VERSION.SDK_INT >= 19) {
-      getWindow().addFlags(WindowManager.LayoutParams.FLAG_LAYOUT_IN_SCREEN|
-                           WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS|
-                           WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR|
-                           WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
-    }
+    getWindow().addFlags(WindowManager.LayoutParams.FLAG_LAYOUT_IN_SCREEN|
+                         WindowManager.LayoutParams.FLAG_LAYOUT_NO_LIMITS|
+                         WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR|
+                         WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);
 
     enableImmersiveModeIfSupported();
 
@@ -224,8 +219,7 @@ public class XCSoar extends Activity {
 
   private void enableImmersiveModeIfSupported() {
     // Set / Reset the System UI visibility flags for Immersive Full Screen Mode, if supported
-    if (android.os.Build.VERSION.SDK_INT >= 19)
-      ImmersiveFullScreenMode.enable(getWindow().getDecorView());
+    ImmersiveFullScreenMode.enable(getWindow().getDecorView());
   }
 
   private static final String[] NEEDED_PERMISSIONS = new String[] {
@@ -265,6 +259,9 @@ public class XCSoar extends Activity {
   @Override protected void onResume() {
     super.onResume();
 
+    if (!Loader.loaded)
+      return;
+
     try {
       startService(new Intent(this, serviceClass));
     } catch (IllegalStateException e) {
@@ -284,6 +281,11 @@ public class XCSoar extends Activity {
 
   @Override protected void onDestroy()
   {
+    if (!Loader.loaded) {
+      super.onDestroy();
+      return;
+    }
+
     Log.d(TAG, "in onDestroy()");
 
     if (batteryReceiver != null) {
@@ -291,8 +293,7 @@ public class XCSoar extends Activity {
       batteryReceiver = null;
     }
 
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.GINGERBREAD)
-      DownloadUtil.Deinitialise(this);
+    DownloadUtil.Deinitialise(this);
 
     if (nativeView != null) {
       nativeView.exitApp();
@@ -330,7 +331,8 @@ public class XCSoar extends Activity {
   }
 
   @Override public void onWindowFocusChanged(boolean hasFocus) {
-    enableImmersiveModeIfSupported();
+    if (Loader.loaded)
+      enableImmersiveModeIfSupported();
     super.onWindowFocusChanged(hasFocus);
   }
 
