@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,12 +22,12 @@ Copyright_License {
 */
 
 #include "TextUtil.hpp"
-#include "Java/Class.hxx"
-#include "Java/String.hxx"
-#include "Java/Exception.hxx"
-#include "Screen/Point.hpp"
+#include "java/Class.hxx"
+#include "java/String.hxx"
+#include "java/Exception.hxx"
+#include "ui/dim/Size.hpp"
 #include "Look/FontDescription.hpp"
-#include "Util/StringView.hxx"
+#include "util/StringView.hxx"
 #include "Asset.hpp"
 
 JNIEnv *TextUtil::env;
@@ -59,7 +59,7 @@ TextUtil::Deinitialise(JNIEnv *env)
 }
 
 TextUtil::TextUtil(jobject _obj)
-  :Java::Object(env, _obj) {
+  :Java::GlobalObject(env, _obj) {
   // get height, ascent_height and capital_height
   assert(midGetFontMetrics);
   jintArray metricsArray = env->NewIntArray(5);
