@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,8 +24,8 @@ Copyright_License {
 #ifndef XCSOAR_ANDROID_TEXT_UTIL_HPP
 #define XCSOAR_ANDROID_TEXT_UTIL_HPP
 
-#include "Java/Object.hxx"
-#include "Compiler.h"
+#include "java/Object.hxx"
+#include "util/Compiler.h"
 
 #include <utility>
 
@@ -33,7 +33,7 @@ struct StringView;
 struct PixelSize;
 class FontDescription;
 
-class TextUtil : protected Java::Object {
+class TextUtil : protected Java::GlobalObject {
   static JNIEnv *env;
   static jmethodID midTextUtil, midGetFontMetrics, midGetTextBounds;
   static jmethodID midGetTextTextureGL;
