@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -46,12 +46,14 @@ GlideComputerBlackboard::StartTask()
   calculated_info.cruise_start_time = gps_info.time;
 
   // JMW reset time cruising/time circling stats on task start
-  calculated_info.time_climb = fixed(0);
-  calculated_info.time_cruise = fixed(0);
-  calculated_info.total_height_gain = fixed(0);
+  calculated_info.time_circling = 0;
+  calculated_info.time_cruise = 0;
+  calculated_info.time_climb_noncircling = 0;
+  calculated_info.time_climb_circling = 0;
+  calculated_info.total_height_gain = 0;
 
   // reset max height gain stuff on task start
-  calculated_info.max_height_gain = fixed(0);
+  calculated_info.max_height_gain = 0;
 }
 
 void
