@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -31,13 +31,13 @@ Copyright_License {
 #include "NMEA/InputLine.hpp"
 #include "Units/System.hpp"
 #include "Waypoint/Waypoint.hpp"
-#include "Util/TruncateString.hpp"
-#include "Util/Macros.hpp"
-#include "Time/TimeoutClock.hpp"
+#include "util/TruncateString.hpp"
+#include "util/Macros.hpp"
+#include "time/TimeoutClock.hpp"
 
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
+#include <cassert>
 #include <tchar.h>
 #ifdef _UNICODE
 #include <windows.h>
@@ -71,9 +71,9 @@ public:
 };
 
 static bool
-ReadAltitude(NMEAInputLine &line, fixed &value_r)
+ReadAltitude(NMEAInputLine &line, double &value_r)
 {
-  fixed value;
+  double value;
   bool available = line.ReadChecked(value);
   char unit = line.ReadFirstChar();
   if (!available)
@@ -108,7 +108,7 @@ PTFRS(NMEAInputLine &line, NMEAInfo &info)
 
   unsigned supply_voltage;
   if (line.ReadChecked(supply_voltage) && supply_voltage != 0) {
-    info.voltage = fixed(supply_voltage) / 1000;
+    info.voltage = supply_voltage / 1000.;
     info.voltage_available.Update(info.clock);
   }
 
@@ -128,7 +128,7 @@ AltairProDevice::ParseNMEA(const char *String, NMEAInfo &info)
   // no propriatary sentence
 
   if (StringIsEqual(type, "$PGRMZ")) {
-    fixed value;
+    double value;
     if (ReadAltitude(line, value))
       info.ProvidePressureAltitude(value);
 
@@ -227,7 +227,7 @@ AltairProDevice::PropertySetGet(char *Buffer, size_t size,
 
   port.Flush();
 
-  TimeoutClock timeout(5000);
+  TimeoutClock timeout(std::chrono::seconds(5));
 
   // eg $PDVSC,S,FOO,BAR*<cr>\r\n
   if (!PortWriteNMEA(port, Buffer, env))
@@ -270,7 +270,7 @@ AltairProDevice::PropertySetGet(TCHAR *s, size_t size,
 {
   assert(s != nullptr);
 
-  char buffer[_tcslen(s) * 4 + 1];
+  char buffer[4096];
   if (::WideCharToMultiByte(CP_ACP, 0, s, -1, buffer, sizeof(buffer),
                                nullptr, nullptr) <= 0)
     return false;
