@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,11 +23,12 @@ Copyright_License {
 
 #include "Device.hpp"
 #include "Device/Port/Port.hpp"
-#include "Util/ConvertString.hpp"
-#include "Util/StaticString.hxx"
-#include "Util/TruncateString.hpp"
-#include "Util/Macros.hpp"
-#include "Util/NumberParser.hpp"
+#include "util/ConvertString.hpp"
+#include "util/StaticString.hxx"
+#include "util/TruncateString.hpp"
+#include "util/Macros.hpp"
+#include "util/NumberParser.hpp"
+#include "util/StringCompare.hxx"
 #include "NMEA/Checksum.hpp"
 
 void
@@ -199,7 +200,8 @@ FlarmDevice::GetConfig(const char *setting, char *buffer, size_t length,
   expected_answer.push_back(',');
 
   Send(request, env);
-  return Receive(expected_answer, buffer, length, env, 2000);
+  return Receive(expected_answer, buffer, length,
+                 env, std::chrono::seconds(2));
 }
 
 /**
@@ -210,7 +212,8 @@ static bool
 ExpectChecksum(Port &port, uint8_t checksum, OperationEnvironment &env)
 {
   char data[4];
-  if (!port.FullRead(data, 3, env, 500) || data[0] != '*')
+  if (!port.FullRead(data, 3, env, std::chrono::milliseconds(500)) ||
+      data[0] != '*')
     return false;
 
   data[3] = '\0';
@@ -228,7 +231,7 @@ FlarmDevice::SetConfig(const char *setting, const char *value,
   expected_answer[6u] = 'A';
 
   Send(buffer, env);
-  return port.ExpectString(expected_answer, env, 2000) &&
+  return port.ExpectString(expected_answer, env, std::chrono::seconds(2)) &&
     ExpectChecksum(port, NMEAChecksum(expected_answer), env);
 }
 
