@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,19 +22,13 @@ Copyright_License {
 */
 
 #include "ConfiguredPort.hpp"
-#include "NullPort.hpp"
-#include "SocketPort.hpp"
+#include "UDPPort.hpp"
 #include "TCPPort.hpp"
 #include "K6BtPort.hpp"
 #include "Device/Config.hpp"
 #include "LogFile.hpp"
-#include "Util/ConvertString.hpp"
-
-#ifdef _WIN32_WCE
-#include "Config/Registry.hpp"
-#else
+#include "util/ConvertString.hpp"
 #include "TCPClientPort.hpp"
-#endif
 
 #ifdef ANDROID
 #include "AndroidBluetoothPort.hpp"
@@ -51,6 +45,9 @@ Copyright_License {
 #include "DumpPort.hpp"
 #endif
 
+#include <stdexcept>
+#include <system_error>
+
 #if defined(HAVE_POSIX) && !defined(ANDROID)
 #include <unistd.h>
 #include <errno.h>
@@ -66,36 +63,28 @@ Copyright_License {
 static bool
 DetectGPS(TCHAR *path, size_t path_max_size)
 {
-#ifdef _WIN32_WCE
-  static const TCHAR *const gps_idm_key =
-    _T("System\\CurrentControlSet\\GPS Intermediate Driver\\Multiplexer");
-  static const TCHAR *const gps_idm_value = _T("DriverInterface");
-
-  RegistryKey key(HKEY_LOCAL_MACHINE, gps_idm_key, true);
-  return !key.error() &&
-    key.GetValue(gps_idm_value, path, path_max_size);
-#else
   return false;
-#endif
 }
 
-static Port *
+static std::unique_ptr<Port>
 WrapPort(const DeviceConfig &config, PortListener *listener,
-         DataHandler &handler, Port *port)
+         DataHandler &handler, std::unique_ptr<Port> port) noexcept
 {
   if (config.k6bt && config.MaybeBluetooth())
-    port = new K6BtPort(port, config.baud_rate, listener, handler);
+    port = std::make_unique<K6BtPort>(std::move(port), config.baud_rate,
+                                      listener, handler);
 
 #ifndef NDEBUG
   if (config.dump_port)
-    port = new DumpPort(port);
+    port = std::make_unique<DumpPort>(std::move(port));
 #endif
 
   return port;
 }
 
-static Port *
-OpenPortInternal(const DeviceConfig &config, PortListener *listener,
+static std::unique_ptr<Port>
+OpenPortInternal(EventLoop &event_loop, Cares::Channel &cares,
+                 const DeviceConfig &config, PortListener *listener,
                  DataHandler &handler)
 {
   const TCHAR *path = nullptr;
@@ -103,55 +92,46 @@ OpenPortInternal(const DeviceConfig &config, PortListener *listener,
 
   switch (config.port_type) {
   case DeviceConfig::PortType::DISABLED:
-    return nullptr;
+    throw std::runtime_error("Port is disabled");
 
   case DeviceConfig::PortType::SERIAL:
     if (config.path.empty())
-      return nullptr;
+      throw std::runtime_error("No port path configured");
 
     path = config.path.c_str();
     break;
 
   case DeviceConfig::PortType::RFCOMM:
 #ifdef ANDROID
-    if (config.bluetooth_mac.empty()) {
-      LogFormat("No Bluetooth MAC configured");
-      return nullptr;
-    }
+    if (config.bluetooth_mac.empty())
+      throw std::runtime_error("No Bluetooth MAC configured");
 
     return OpenAndroidBluetoothPort(config.bluetooth_mac, listener, handler);
 #else
-    LogFormat("Bluetooth not available on this platform");
-    return nullptr;
+    throw std::runtime_error("Bluetooth not available");
 #endif
 
   case DeviceConfig::PortType::RFCOMM_SERVER:
 #ifdef ANDROID
     return OpenAndroidBluetoothServerPort(listener, handler);
 #else
-    LogFormat("Bluetooth not available on this platform");
-    return nullptr;
+    throw std::runtime_error("Bluetooth not available");
 #endif
 
   case DeviceConfig::PortType::IOIOUART:
 #if defined(ANDROID)
-    if (config.ioio_uart_id >= AndroidIOIOUartPort::getNumberUarts()) {
-      LogFormat("No IOIOUart configured in profile");
-      return nullptr;
-    }
+    if (config.ioio_uart_id >= AndroidIOIOUartPort::getNumberUarts())
+      throw std::runtime_error("No IOIOUart configured in profile");
 
     return OpenAndroidIOIOUartPort(config.ioio_uart_id, config.baud_rate,
                                    listener, handler);
 #else
-    LogFormat("IOIO Uart not available on this platform or version");
-    return nullptr;
+    throw std::runtime_error("IOIO driver not available");
 #endif
 
   case DeviceConfig::PortType::AUTO:
-    if (!DetectGPS(buffer, sizeof(buffer))) {
-      LogFormat("no GPS detected");
-      return nullptr;
-    }
+    if (!DetectGPS(buffer, sizeof(buffer)))
+      throw std::runtime_error("No GPS detected");
 
     LogFormat(_T("GPS detected: %s"), buffer);
 
@@ -163,95 +143,71 @@ OpenPortInternal(const DeviceConfig &config, PortListener *listener,
   case DeviceConfig::PortType::NUNCHUCK:
   case DeviceConfig::PortType::I2CPRESSURESENSOR:
   case DeviceConfig::PortType::IOIOVOLTAGE:
+  case DeviceConfig::PortType::GLIDER_LINK:
     break;
 
   case DeviceConfig::PortType::TCP_CLIENT: {
-#ifdef _WIN32_WCE
-    return nullptr;
-#else
     const WideToUTF8Converter ip_address(config.ip_address);
     if (!ip_address.IsValid())
-      return nullptr;
+      throw std::runtime_error("No IP address configured");
 
-    auto port = new TCPClientPort(listener, handler);
-    if (!port->Connect(ip_address, config.tcp_port)) {
-      delete port;
-      return nullptr;
-    }
-
-    return port;
-#endif
+    return std::make_unique<TCPClientPort>(event_loop, cares,
+                                           ip_address, config.tcp_port,
+                                           listener, handler);
   }
 
-  case DeviceConfig::PortType::TCP_LISTENER: {
-    TCPPort *port = new TCPPort(listener, handler);
-    if (!port->Open(config.tcp_port)) {
-      delete port;
-      return nullptr;
-    }
+  case DeviceConfig::PortType::TCP_LISTENER:
+    return std::make_unique<TCPPort>(event_loop, config.tcp_port,
+                                     listener, handler);
 
-    return port;
-  }
-
-  case DeviceConfig::PortType::UDP_LISTENER: {
-    SocketPort *port = new SocketPort(listener, handler);
-    if (!port->OpenUDPListener(config.tcp_port)) {
-      delete port;
-      return nullptr;
-    }
-
-    return port;
-  }
+  case DeviceConfig::PortType::UDP_LISTENER:
+    return std::make_unique<UDPPort>(event_loop, config.tcp_port,
+                                     listener, handler);
 
   case DeviceConfig::PortType::PTY: {
 #if defined(HAVE_POSIX) && !defined(ANDROID)
     if (config.path.empty())
-      return nullptr;
+      throw std::runtime_error("No pty path configured");
 
     if (unlink(config.path.c_str()) < 0 && errno != ENOENT)
-      return nullptr;
+      throw std::system_error(std::error_code(errno,
+                                              std::system_category()),
+                              "Failed to delete pty");
 
-    TTYPort *port = new TTYPort(listener, handler);
+    auto port = std::make_unique<TTYPort>(event_loop, listener, handler);
     const char *slave_path = port->OpenPseudo();
-    if (slave_path == nullptr) {
-      delete port;
-      return nullptr;
-    }
 
-    if (symlink(slave_path, config.path.c_str()) < 0) {
-      delete port;
-      return nullptr;
-    }
+    if (symlink(slave_path, config.path.c_str()) < 0)
+      throw std::system_error(std::error_code(errno,
+                                              std::system_category()),
+                              "Failed to symlink pty");
 
     return port;
 #else
-    return nullptr;
+    throw std::runtime_error("Pty not available");
 #endif
   }
   }
 
   if (path == nullptr)
-    return nullptr;
+    throw std::runtime_error("No port path configured");
 
 #ifdef HAVE_POSIX
-  TTYPort *port = new TTYPort(listener, handler);
+  auto port = std::make_unique<TTYPort>(event_loop, listener, handler);
 #else
-  SerialPort *port = new SerialPort(listener, handler);
+  auto port = std::make_unique<SerialPort>(listener, handler);
 #endif
-  if (!port->Open(path, config.baud_rate)) {
-    delete port;
-    return nullptr;
-  }
-
+  port->Open(path, config.baud_rate);
   return port;
 }
 
-Port *
-OpenPort(const DeviceConfig &config, PortListener *listener,
+std::unique_ptr<Port>
+OpenPort(EventLoop &event_loop, Cares::Channel &cares,
+         const DeviceConfig &config, PortListener *listener,
          DataHandler &handler)
 {
-  Port *port = OpenPortInternal(config, listener, handler);
+  auto port = OpenPortInternal(event_loop, cares, config, listener, handler);
   if (port != nullptr)
-    port = WrapPort(config, listener, handler, port);
+    port = WrapPort(config, listener, handler, std::move(port));
   return port;
 }
