@@ -2,7 +2,7 @@
   Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,9 +24,13 @@
 #ifndef XCSOAR_SCAN_BLUETOOTH_LE_DIALOG_HPP
 #define XCSOAR_SCAN_BLUETOOTH_LE_DIALOG_HPP
 
-#include <stddef.h>
+#include <string>
 
-bool
-ScanBluetoothLeDialog(char *address, size_t address_size);
+/**
+ * @return the selected address or an empty string if the user has
+ * canceled the dialog
+ */
+std::string
+ScanBluetoothLeDialog() noexcept;
 
 #endif
