@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -47,26 +47,27 @@ public:
     :RowFormWidget(UIGlobals::GetDialogLook()) {}
 
 public:
-  virtual void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
-  virtual bool Save(bool &changed) override;
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
+  bool Save(bool &changed) noexcept override;
 };
 
 void
-TaskRulesConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
+TaskRulesConfigPanel::Prepare(ContainerWindow &parent,
+                              const PixelRect &rc) noexcept
 {
   const ComputerSettings &settings_computer = CommonInterface::GetComputerSettings();
   const TaskBehaviour &task_behaviour = settings_computer.task;
 
   RowFormWidget::Prepare(parent, rc);
 
-  AddFloat(_("Start max. speed"), _("Maximum speed allowed in start observation zone.  Set to 0 for no limit."),
-           _T("%.0f %s"), _T("%.0f"), fixed(0), fixed(300), fixed(5), false, UnitGroup::HORIZONTAL_SPEED,
+  AddFloat(_("Start max. speed"), _("Maximum speed allowed in start observation zone. Set to 0 for no limit."),
+           _T("%.0f %s"), _T("%.0f"), 0, 300, 5, false, UnitGroup::HORIZONTAL_SPEED,
            task_behaviour.ordered_defaults.start_constraints.max_speed);
   SetExpertRow(StartMaxSpeed);
 
   AddFloat(_("Start max. speed margin"),
            _("Maximum speed above maximum start speed to tolerate.  Set to 0 for no tolerance."),
-           _T("%.0f %s"), _T("%.0f"), fixed(0), fixed(300), fixed(5), false, UnitGroup::HORIZONTAL_SPEED,
+           _T("%.0f %s"), _T("%.0f"), 0, 300, 5, false, UnitGroup::HORIZONTAL_SPEED,
            task_behaviour.start_margins.max_speed_margin);
   SetExpertRow(StartMaxSpeedMargin);
 
@@ -76,14 +77,14 @@ TaskRulesConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   AddFloat(_("Start max. height"),
            _("Maximum height based on start height reference (AGL or MSL) while starting the task.  "
                "Set to 0 for no limit."),
-           _T("%.0f %s"), _T("%.0f"), fixed(0), fixed(10000), fixed(50), false, UnitGroup::ALTITUDE,
-           fixed(task_behaviour.ordered_defaults.start_constraints.max_height));
+           _T("%.0f %s"), _T("%.0f"), 0, 10000, 50, false, UnitGroup::ALTITUDE,
+           task_behaviour.ordered_defaults.start_constraints.max_height);
   SetExpertRow(StartMaxHeight);
 
   AddFloat(_("Start max. height margin"),
            _("Maximum height above maximum start height to tolerate.  Set to 0 for no tolerance."),
-           _T("%.0f %s"), _T("%.0f"), fixed(0), fixed(10000), fixed(50), false, UnitGroup::ALTITUDE,
-           fixed(task_behaviour.start_margins.max_height_margin));
+           _T("%.0f %s"), _T("%.0f"), 0, 10000, 50, false, UnitGroup::ALTITUDE,
+           task_behaviour.start_margins.max_height_margin);
   SetExpertRow(StartMaxHeightMargin);
 
   static constexpr StaticEnumChoice altitude_reference_list[] = {
@@ -106,8 +107,8 @@ TaskRulesConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   AddFloat(_("Finish min. height"),
            _("Minimum height based on finish height reference (AGL or MSL) while finishing the task.  "
                "Set to 0 for no limit."),
-           _T("%.0f %s"), _T("%.0f"), fixed(0), fixed(10000), fixed(50), false, UnitGroup::ALTITUDE,
-           fixed(task_behaviour.ordered_defaults.finish_constraints.min_height));
+           _T("%.0f %s"), _T("%.0f"), 0, 10000, 50, false, UnitGroup::ALTITUDE,
+           task_behaviour.ordered_defaults.finish_constraints.min_height);
   SetExpertRow(FinishMinHeight);
 
   AddEnum(_("Finish height ref."),
@@ -119,7 +120,7 @@ TaskRulesConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
 
 
 bool
-TaskRulesConfigPanel::Save(bool &_changed)
+TaskRulesConfigPanel::Save(bool &_changed) noexcept
 {
   bool changed = false;
 
@@ -156,8 +157,8 @@ TaskRulesConfigPanel::Save(bool &_changed)
   return true;
 }
 
-Widget *
+std::unique_ptr<Widget>
 CreateTaskRulesConfigPanel()
 {
-  return new TaskRulesConfigPanel();
+  return std::make_unique<TaskRulesConfigPanel>();
 }
