@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -30,10 +30,10 @@ class StatusPanel : public RowFormWidget {
 public:
   StatusPanel(const DialogLook &look):RowFormWidget(look) {}
 
-  virtual void Refresh() = 0;
+  virtual void Refresh() noexcept = 0;
 
   /* virtual methods from class Widget */
-  void Show(const PixelRect &rc) override;
+  void Show(const PixelRect &rc) noexcept override;
 };
 
 #endif
