@@ -2,7 +2,7 @@
   Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@
  */
 
 #include "TrafficDialogs.hpp"
+#include "Look/TrafficLook.hpp"
 #include "Dialogs/TextEntry.hpp"
 #include "Dialogs/Message.hpp"
 #include "Dialogs/WidgetDialog.hpp"
@@ -40,27 +41,21 @@
 #include "FLARM/Friends.hpp"
 #include "FLARM/Glue.hpp"
 #include "Renderer/ColorButtonRenderer.hpp"
-#include "Renderer/ColorButtonRenderer.hpp"
-#include "Screen/Layout.hpp"
-#include "Geo/Math.hpp"
 #include "UIGlobals.hpp"
 #include "Components.hpp"
 #include "Formatter/UserUnits.hpp"
 #include "Formatter/AngleFormatter.hpp"
-#include "Util/StringBuilder.hxx"
-#include "Util/StringUtil.hpp"
-#include "Util/Macros.hpp"
+#include "util/StringBuilder.hxx"
+#include "util/StringCompare.hxx"
+#include "util/Macros.hpp"
 #include "Language/Language.hpp"
 #include "Interface.hpp"
 #include "Blackboard/LiveBlackboard.hpp"
 #include "Blackboard/BlackboardListener.hpp"
 #include "TeamActions.hpp"
 
-#include <math.h>
-#include <stdio.h>
-
 class FlarmTrafficDetailsWidget final
-  : public RowFormWidget, ActionListener, NullBlackboardListener {
+  : public RowFormWidget, NullBlackboardListener {
   enum Controls {
     CALLSIGN,
     CHANGE_CALLSIGN_BUTTON,
@@ -75,16 +70,6 @@ class FlarmTrafficDetailsWidget final
     PLANE,
   };
 
-  enum Buttons {
-    CHANGE_CALLSIGN,
-    TEAM,
-    CLEAR,
-    GREEN,
-    BLUE,
-    YELLOW,
-    MAGENTA,
-  };
-
   WndForm &dialog;
 
   const FlarmId target_id;
@@ -97,9 +82,9 @@ public:
   void CreateButtons(WidgetDialog &buttons);
 
   /* virtual methods from Widget */
-  void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
-  void Show(const PixelRect &rc) override;
-  void Hide() override;
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
+  void Show(const PixelRect &rc) noexcept override;
+  void Hide() noexcept override;
 
 private:
   void UpdateChanging(const MoreData &basic);
@@ -109,9 +94,6 @@ private:
   void OnTeamClicked();
   void OnFriendColorClicked(FlarmColor color);
 
-  /* virtual methods from ActionListener */
-  void OnAction(int id) override;
-
   /* virtual methods from BlackboardListener */
   void OnGPSUpdate(const MoreData &basic) override {
     UpdateChanging(basic);
@@ -121,30 +103,34 @@ private:
 inline void
 FlarmTrafficDetailsWidget::CreateButtons(WidgetDialog &buttons)
 {
-  const ButtonLook &look = buttons.GetButtonLook();
+  const ButtonLook &button_look = buttons.GetButtonLook();
 
-  constexpr Color green(0x74, 0xff, 0x00);
-  buttons.AddButton(new ColorButtonRenderer(look, green), *this, GREEN);
+  buttons.AddButton(std::make_unique<ColorButtonRenderer>(button_look,
+                                                          TrafficLook::team_color_green),
+                    [this](){ OnFriendColorClicked(FlarmColor::GREEN); });
 
-  constexpr Color blue(0x00, 0x90, 0xff);
-  buttons.AddButton(new ColorButtonRenderer(look, blue), *this, BLUE);
+  buttons.AddButton(std::make_unique<ColorButtonRenderer>(button_look,
+                                                          TrafficLook::team_color_blue),
+                    [this](){ OnFriendColorClicked(FlarmColor::BLUE); });
 
-  constexpr Color yellow(0xff, 0xe8, 0x00);
-  buttons.AddButton(new ColorButtonRenderer(look, yellow), *this, YELLOW);
+  buttons.AddButton(std::make_unique<ColorButtonRenderer>(button_look,
+                                                          TrafficLook::team_color_yellow),
+                    [this](){ OnFriendColorClicked(FlarmColor::YELLOW); });
 
-  constexpr Color magenta(0xff, 0x00, 0xcb);
-  buttons.AddButton(new ColorButtonRenderer(look, magenta), *this, MAGENTA);
+  buttons.AddButton(std::make_unique<ColorButtonRenderer>(button_look,
+                                                          TrafficLook::team_color_magenta),
+                    [this](){ OnFriendColorClicked(FlarmColor::MAGENTA); });
 
-  buttons.AddButton(_("Clear"), *this, CLEAR);
-  buttons.AddButton(_("Team"), *this, TEAM);
+  buttons.AddButton(_("Clear"), [this](){ OnFriendColorClicked(FlarmColor::NONE); });
+  buttons.AddButton(_("Team"), [this](){ OnTeamClicked(); });
 }
 
 void
 FlarmTrafficDetailsWidget::Prepare(ContainerWindow &parent,
-                                   const PixelRect &rc)
+                                   const PixelRect &rc) noexcept
 {
   AddReadOnly(_("Callsign"));
-  AddButton(_("Change callsign"), *this, CHANGE_CALLSIGN);
+  AddButton(_("Change callsign"), [this](){ OnCallsignClicked(); });
   AddSpacer();
   AddReadOnly(_("Distance"));
   AddReadOnly(_("Altitude"));
@@ -159,7 +145,7 @@ FlarmTrafficDetailsWidget::Prepare(ContainerWindow &parent,
 }
 
 void
-FlarmTrafficDetailsWidget::Show(const PixelRect &rc)
+FlarmTrafficDetailsWidget::Show(const PixelRect &rc) noexcept
 {
   RowFormWidget::Show(rc);
   Update();
@@ -167,7 +153,7 @@ FlarmTrafficDetailsWidget::Show(const PixelRect &rc)
 }
 
 void
-FlarmTrafficDetailsWidget::Hide()
+FlarmTrafficDetailsWidget::Hide() noexcept
 {
   CommonInterface::GetLiveBlackboard().RemoveListener(*this);
   RowFormWidget::Hide();
@@ -190,7 +176,7 @@ FlarmTrafficDetailsWidget::UpdateChanging(const MoreData &basic)
 
   // Fill distance/direction field
   if (target_ok) {
-    FormatUserDistanceSmart(target->distance, tmp, 20, fixed(1000));
+    FormatUserDistanceSmart(target->distance, tmp, 20, 1000);
     TCHAR *p = tmp + _tcslen(tmp);
     *p++ = _T(' ');
     FormatAngleDelta(p, 20, target->Bearing() - basic.track);
@@ -342,40 +328,6 @@ FlarmTrafficDetailsWidget::OnFriendColorClicked(FlarmColor color)
   dialog.SetModalResult(mrOK);
 }
 
-void
-FlarmTrafficDetailsWidget::OnAction(int id)
-{
-  switch (id) {
-  case CHANGE_CALLSIGN:
-    OnCallsignClicked();
-    break;
-
-  case TEAM:
-    OnTeamClicked();
-    break;
-
-  case CLEAR:
-    OnFriendColorClicked(FlarmColor::NONE);
-    break;
-
-  case GREEN:
-    OnFriendColorClicked(FlarmColor::GREEN);
-    break;
-
-  case BLUE:
-    OnFriendColorClicked(FlarmColor::BLUE);
-    break;
-
-  case YELLOW:
-    OnFriendColorClicked(FlarmColor::YELLOW);
-    break;
-
-  case MAGENTA:
-    OnFriendColorClicked(FlarmColor::MAGENTA);
-    break;
-  }
-}
-
 /**
  * The function opens the FLARM Traffic Details dialog
  */
@@ -384,13 +336,13 @@ dlgFlarmTrafficDetailsShowModal(FlarmId id)
 {
   const DialogLook &look = UIGlobals::GetDialogLook();
 
-  WidgetDialog dialog(look);
+  WidgetDialog dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+                      look, _("FLARM Traffic Details"));
 
   FlarmTrafficDetailsWidget *widget =
     new FlarmTrafficDetailsWidget(dialog, id);
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("FLARM Traffic Details"),
-                    widget);
   widget->CreateButtons(dialog);
   dialog.AddButton(_("Close"), mrCancel);
+  dialog.FinishPreliminary(widget);
   dialog.ShowModal();
 }
