@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,15 +22,15 @@
 
 #include "NetCoupe.hpp"
 
-NetCoupe::NetCoupe(const Trace &_trace)
+NetCoupe::NetCoupe(const Trace &_trace) noexcept
   :ContestDijkstra(_trace, true, 4, 1000) {}
 
 ContestResult
-NetCoupe::CalculateResult() const
+NetCoupe::CalculateResult() const noexcept
 {
   ContestResult result = ContestDijkstra::CalculateResult();
   // 0.8 factor for free distance and 1/1000 m -> km
-  result.score = ApplyHandicap(result.distance * fixed(0.0008));
+  result.score = ApplyHandicap(result.distance * 0.0008);
   return result;
 }
 
