@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,15 +25,15 @@ Copyright_License {
 #define XCSOAR_TASK_FACTORY_CREATE_HPP
 
 #include "TaskFactoryType.hpp"
-#include "Compiler.h"
+
+#include <memory>
 
 class AbstractTaskFactory;
 class OrderedTask;
 struct TaskBehaviour;
 
-gcc_malloc
-AbstractTaskFactory *
+std::unique_ptr<AbstractTaskFactory>
 CreateTaskFactory(TaskFactoryType type, OrderedTask &task,
-                  const TaskBehaviour &task_behaviour);
+                  const TaskBehaviour &task_behaviour) noexcept;
 
 #endif
