@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,18 +25,17 @@ Copyright_License {
 #define TRACE_HPP
 
 #include "Point.hpp"
-#include "Util/NonCopyable.hpp"
-#include "Util/SliceAllocator.hxx"
-#include "Util/Serial.hpp"
+#include "util/NonCopyable.hpp"
+#include "util/SliceAllocator.hxx"
+#include "util/Serial.hpp"
 #include "Geo/Flat/TaskProjection.hpp"
-#include "Compiler.h"
 
 #include <boost/intrusive/list.hpp>
 #include <boost/intrusive/set.hpp>
 
 #include <algorithm>
 
-#include <assert.h>
+#include <cassert>
 #include <stdlib.h>
 
 class TracePointVector;
@@ -64,7 +63,7 @@ class Trace : private NonCopyable
      * time delta.
      * This is like a modified Douglas-Peuker algorithm
      */
-    gcc_pure
+    [[gnu::pure]]
     static bool DeltaRank(const TraceDelta &x, const TraceDelta &y) {
       // distance is king
       if (x.elim_distance < y.elim_distance)
@@ -91,7 +90,7 @@ class Trace : private NonCopyable
     }
 
     struct DeltaRankOp {
-      gcc_pure
+      [[gnu::pure]]
       bool operator()(const TraceDelta &s1, const TraceDelta &s2) const {
         return DeltaRank(s1, s2);
       }
@@ -240,7 +239,7 @@ protected:
    *
    * @return Recent time
    */
-  gcc_pure
+  [[gnu::pure]]
   unsigned GetRecentTime(const unsigned t) const;
 
   /**
@@ -317,11 +316,11 @@ public:
    */
   void clear();
 
-  void EraseEarlierThan(fixed time) {
+  void EraseEarlierThan(double time) {
     EraseEarlierThan((unsigned)time);
   }
 
-  void EraseLaterThan(fixed time) {
+  void EraseLaterThan(double time) {
     EraseLaterThan((unsigned)time);
   }
 
@@ -395,7 +394,7 @@ public:
    * resolution #min_distance.
    */
   void GetPoints(TracePointVector &v, unsigned min_time,
-                 const GeoPoint &location, fixed resolution) const;
+                 const GeoPoint &location, double resolution) const;
 
   const TracePoint &front() const {
     assert(!empty());
@@ -445,10 +444,10 @@ private:
     return chronological_list.back();
   }
 
-  gcc_pure
+  [[gnu::pure]]
   unsigned CalcAverageDeltaDistance(const unsigned no_thin) const;
 
-  gcc_pure
+  [[gnu::pure]]
   unsigned CalcAverageDeltaTime(const unsigned no_thin) const;
 
   static constexpr unsigned null_delta = 0 - 1;
@@ -518,8 +517,8 @@ public:
     return task_projection;
   }
 
-  gcc_pure
-  unsigned ProjectRange(const GeoPoint &location, fixed distance) const {
+  [[gnu::pure]]
+  unsigned ProjectRange(const GeoPoint &location, double distance) const {
     return task_projection.ProjectRangeInteger(location, distance);
   }
 };
