@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,12 +25,10 @@ Copyright_License {
 #define XCSOAR_DATA_FIELD_ENUM_HPP
 
 #include "Base.hpp"
-#include "Util/StaticArray.hpp"
+#include "util/StaticArray.hxx"
 
 #include <utility>
 
-#include <stdlib.h>
-
 /**
  * A struct that is used for static initialisation of the enum list.
  */
@@ -49,18 +47,18 @@ public:
     TCHAR *help;
 
   public:
-    Entry():string(nullptr), display_string(nullptr), help(nullptr) {}
-    ~Entry();
+    Entry() noexcept:string(nullptr), display_string(nullptr), help(nullptr) {}
+    ~Entry() noexcept;
 
     Entry(const Entry &) = delete;
 
-    Entry(Entry &&other)
+    Entry(Entry &&other) noexcept
       :id(other.id), string(other.string),
        display_string(other.display_string), help(other.help) {
       other.string = other.display_string = other.help = nullptr;
     }
 
-    Entry &operator=(Entry &&other) {
+    Entry &operator=(Entry &&other) noexcept {
       id = other.id;
       std::swap(string, other.string);
       std::swap(display_string, other.display_string);
@@ -68,33 +66,33 @@ public:
       return *this;
     }
 
-    friend void swap(Entry &a, Entry &b) {
+    friend void swap(Entry &a, Entry &b) noexcept {
       std::swap(a.id, b.id);
       std::swap(a.string, b.string);
       std::swap(a.display_string, b.display_string);
       std::swap(a.help, b.help);
     }
 
-    unsigned GetId() const {
+    unsigned GetId() const noexcept {
       return id;
     }
 
-    const TCHAR *GetString() const {
+    const TCHAR *GetString() const noexcept {
       return string;
     }
 
-    const TCHAR *GetDisplayString() const {
+    const TCHAR *GetDisplayString() const noexcept {
       return display_string;
     }
 
-    const TCHAR *GetHelp() const {
+    const TCHAR *GetHelp() const noexcept {
       return help;
     }
 
-    void SetString(const TCHAR *_string);
+    void SetString(const TCHAR *_string) noexcept;
     void Set(unsigned _id, const TCHAR *_string,
              const TCHAR *_display_string=nullptr,
-             const TCHAR *_help=nullptr);
+             const TCHAR *_help=nullptr) noexcept;
   };
 
 private:
@@ -102,61 +100,62 @@ private:
   unsigned int value;
 
 public:
-  DataFieldEnum(DataFieldListener *listener=nullptr)
+  DataFieldEnum(DataFieldListener *listener=nullptr) noexcept
     :DataField(Type::ENUM, true, listener), value(0) {}
 
   gcc_pure
-  unsigned GetValue() const;
+  unsigned GetValue() const noexcept;
 
   gcc_pure
-  bool Exists(const TCHAR *text) const {
+  bool Exists(const TCHAR *text) const noexcept {
     return Find(text) >= 0;
   }
 
-  void replaceEnumText(unsigned int i, const TCHAR *Text);
+  void replaceEnumText(unsigned int i, const TCHAR *Text) noexcept;
 
   /**
    * Clear the list of choices.  This will not notify the
    * DataFieldListener.
    */
-  void ClearChoices() {
+  void ClearChoices() noexcept {
     entries.clear();
     value = 0;
   }
 
   bool AddChoice(unsigned id, const TCHAR *text,
                  const TCHAR *display_string=nullptr,
-                 const TCHAR *help=nullptr);
+                 const TCHAR *help=nullptr) noexcept;
 
   /**
    * Add choices from the specified nullptr-terminated list (the last
    * entry has a nullptr display_string).  All display strings and help
    * texts are translated with gettext() by this method.
    */
-  void AddChoices(const StaticEnumChoice *list);
+  void AddChoices(const StaticEnumChoice *list) noexcept;
 
-  bool addEnumText(const TCHAR *text, unsigned id, const TCHAR *help=nullptr) {
+  bool addEnumText(const TCHAR *text, unsigned id,
+                   const TCHAR *help=nullptr) noexcept {
     return AddChoice(id, text, nullptr, help);
   }
 
   unsigned addEnumText(const TCHAR *Text, const TCHAR *display_string=nullptr,
-                       const TCHAR *ItemHelpText=nullptr);
-  void addEnumTexts(const TCHAR *const*list);
+                       const TCHAR *ItemHelpText=nullptr) noexcept;
+  void addEnumTexts(const TCHAR *const*list) noexcept;
 
   /**
    * @return help of current enum item or nullptr if current item has no help
    */
-  const TCHAR *GetHelp() const;
+  const TCHAR *GetHelp() const noexcept;
 
   /**
    * @param value True if display item help in text box below picker
    * Displays help strings associated with enums Items
    */
-  void EnableItemHelp(bool value) override {
+  void EnableItemHelp(bool value) noexcept override {
     item_help_enabled = value;
   }
 
-  void Set(unsigned Value);
+  void Set(unsigned Value) noexcept;
 
   /**
    * Select the item with the specified text (not display string).
@@ -165,7 +164,7 @@ public:
    * @return false if an item with the specified text was not found,
    * and therefore the value was not changed
    */
-  bool Set(const TCHAR *text);
+  bool Set(const TCHAR *text) noexcept;
 
   /**
    * Set the value to the specified string.  If there is no choice
@@ -173,37 +172,38 @@ public:
    *
    * @return the new integer value
    */
-  int SetStringAutoAdd(const TCHAR *text);
+  int SetStringAutoAdd(const TCHAR *text) noexcept;
 
-  void Sort(unsigned startindex = 0);
+  void Sort(unsigned startindex = 0) noexcept;
 
   gcc_pure
-  unsigned Count() const {
+  unsigned Count() const noexcept {
     return entries.size();
   }
-  unsigned getItem(unsigned index) const;
+
+  unsigned getItem(unsigned index) const noexcept;
 
   /* virtual methods from class DataField */
-  void Inc() override;
-  void Dec() override;
-  int GetAsInteger() const override;
-  const TCHAR *GetAsString() const override;
-  const TCHAR *GetAsDisplayString() const override;
-  void SetAsInteger(int value) override;
-  void SetAsString(const TCHAR *value) override;
-  ComboList CreateComboList(const TCHAR *reference) const override;
+  void Inc() noexcept override;
+  void Dec() noexcept override;
+  int GetAsInteger() const noexcept override;
+  const TCHAR *GetAsString() const noexcept override;
+  const TCHAR *GetAsDisplayString() const noexcept override;
+  void SetAsInteger(int value) noexcept override;
+  void SetAsString(const TCHAR *value) noexcept override;
+  ComboList CreateComboList(const TCHAR *reference) const noexcept override;
 
 protected:
   /**
    * Finds an entry with the specified text.  Returns -1 if not found.
    */
   gcc_pure
-  int Find(const TCHAR *text) const;
+  int Find(const TCHAR *text) const noexcept;
 
   gcc_pure
-  int Find(unsigned id) const;
+  int Find(unsigned id) const noexcept;
 
-  void SetIndex(unsigned new_value, bool invoke_callback);
+  void SetIndex(unsigned new_value, bool invoke_callback) noexcept;
 };
 
 #endif
