@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@ Copyright_License {
 #include "String.hpp"
 
 void
-DataFieldString::SetAsString(const TCHAR *Value)
+DataFieldString::SetAsString(const TCHAR *Value) noexcept
 {
   if (mValue == Value)
     return;
@@ -34,13 +34,13 @@ DataFieldString::SetAsString(const TCHAR *Value)
 }
 
 void
-DataFieldString::Set(const TCHAR *Value)
+DataFieldString::Set(const TCHAR *Value) noexcept
 {
   mValue = Value;
 }
 
 const TCHAR *
-DataFieldString::GetAsString() const
+DataFieldString::GetAsString() const noexcept
 {
   return mValue;
 }
