@@ -2,7 +2,7 @@
   Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,22 +24,22 @@
 #include "FlarmTrafficWindow.hpp"
 #include "FLARM/Traffic.hpp"
 #include "FLARM/Friends.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "Formatter/UserUnits.hpp"
 #include "Units/Units.hpp"
 #include "Math/Screen.hpp"
 #include "Language/Language.hpp"
-#include "Util/Macros.hpp"
+#include "util/Macros.hpp"
 #include "Look/FlarmTrafficLook.hpp"
 
 #include <algorithm>
 
-#include <assert.h>
+#include <cassert>
 #include <stdio.h>
 
 #ifdef ENABLE_OPENGL
-#include "Screen/OpenGL/Scope.hpp"
+#include "ui/canvas/opengl/Scope.hpp"
 #endif
 
 
@@ -75,8 +75,8 @@ FlarmTrafficWindow::OnResize(PixelSize new_size)
 {
   PaintWindow::OnResize(new_size);
 
-  const unsigned half_width = new_size.cx / 2;
-  const unsigned half_height = new_size.cy / 2;
+  const unsigned half_width = new_size.width / 2;
+  const unsigned half_height = new_size.height / 2;
 
   // Calculate Radar size
   radius = std::min(half_width - h_padding, half_height - v_padding);
@@ -150,7 +150,7 @@ FlarmTrafficWindow::PrevTarget()
  * to select the next one
  */
 void
-FlarmTrafficWindow::UpdateSelector(const FlarmId id, const RasterPoint pt)
+FlarmTrafficWindow::UpdateSelector(const FlarmId id, const PixelPoint pt)
 {
   // Update #selection
   if (!id.IsDefined())
@@ -159,11 +159,11 @@ FlarmTrafficWindow::UpdateSelector(const FlarmId id, const RasterPoint pt)
     SetTarget(id);
 
   // If we don't have a valid selection and we can't find
-  // a target close to to the RasterPoint we select the next one
+  // a target close to to the PixelPoint we select the next one
   // on the internal list
   if (selection < 0 && (
       pt.x < 0 || pt.y < 0 ||
-      !SelectNearTarget(pt.x, pt.y, radius * 2)) )
+      !SelectNearTarget(pt, radius * 2)) )
     NextTarget();
 }
 
@@ -194,7 +194,7 @@ FlarmTrafficWindow::Update(Angle new_direction, const TrafficList &new_data,
     return;
 
   FlarmId selection_id;
-  RasterPoint pt;
+  PixelPoint pt;
   if (!small && selection >= 0) {
     selection_id = data.list[selection].id;
     pt = sc[selection];
@@ -206,8 +206,8 @@ FlarmTrafficWindow::Update(Angle new_direction, const TrafficList &new_data,
 
   data_modified = new_data.modified;
   heading = new_direction;
-  fr.SetAngle(-heading);
-  fir.SetAngle(heading);
+  fr = -heading;
+  fir = heading;
   data = new_data;
   settings = new_settings;
 
@@ -221,11 +221,11 @@ FlarmTrafficWindow::Update(Angle new_direction, const TrafficList &new_data,
  * Returns the distance to the own plane in pixels
  * @param d Distance in meters to the own plane
  */
-fixed
-FlarmTrafficWindow::RangeScale(fixed d) const
+double
+FlarmTrafficWindow::RangeScale(double d) const
 {
-  d = d / distance;
-  return std::min(d, fixed(1)) * radius;
+  d /= distance;
+  return std::min(d, 1.) * radius;
 }
 
 /**
@@ -242,7 +242,7 @@ FlarmTrafficWindow::PaintRadarNoTraffic(Canvas &canvas) const
   canvas.Select(look.no_traffic_font);
   PixelSize ts = canvas.CalcTextSize(str);
   canvas.SetTextColor(look.default_color);
-  canvas.DrawText(radar_mid.x - (ts.cx / 2), radar_mid.y - (radius / 2), str);
+  canvas.DrawText(radar_mid - PixelSize{ts.width / 2, radius / 2}, str);
 }
 
 gcc_const
@@ -283,14 +283,13 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
                                      unsigned i)
 {
   // Save relative East/North
-  Point2D<fixed> p(traffic.relative_east,
-                   -traffic.relative_north);
+  DoublePoint2D p(traffic.relative_east, -traffic.relative_north);
 
   // Calculate the distance in pixels
-  fixed scale = RangeScale(traffic.distance);
+  double scale = RangeScale(traffic.distance);
 
   // Don't display distracting, far away targets in WarningMode
-  if (WarningMode() && !traffic.HasAlarm() && scale == fixed(radius))
+  if (WarningMode() && !traffic.HasAlarm() && scale == radius)
     return;
 
   // x and y are not between 0 and 1 (distance will be handled via scale)
@@ -298,8 +297,8 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
     p.x /= traffic.distance;
     p.y /= traffic.distance;
   } else {
-    p.x = fixed(0);
-    p.y = fixed(0);
+    p.x = 0;
+    p.y = 0;
   }
 
   if (!enable_north_up) {
@@ -378,13 +377,13 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
   if (circles > 0) {
     canvas.SelectHollowBrush();
     canvas.Select(*circle_pen);
-    canvas.DrawCircle(sc[i].x, sc[i].y, Layout::FastScale(small ? 8 : 16));
+    canvas.DrawCircle(sc[i], Layout::FastScale(small ? 8 : 16));
     if (circles == 2)
-      canvas.DrawCircle(sc[i].x, sc[i].y, Layout::FastScale(small ? 10 : 19));
+      canvas.DrawCircle(sc[i], Layout::FastScale(small ? 10 : 19));
   }
 
   // Create an arrow polygon
-  RasterPoint Arrow[5];
+  BulkPixelPoint Arrow[4];
   if (small) {
     Arrow[0].x = -3;
     Arrow[0].y = 4;
@@ -394,8 +393,6 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
     Arrow[2].y = 4;
     Arrow[3].x = 0;
     Arrow[3].y = 2;
-    Arrow[4].x = -3;
-    Arrow[4].y = 4;
   } else {
     Arrow[0].x = -6;
     Arrow[0].y = 8;
@@ -405,12 +402,10 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
     Arrow[2].y = 8;
     Arrow[3].x = 0;
     Arrow[3].y = 5;
-    Arrow[4].x = -6;
-    Arrow[4].y = 8;
   }
 
   // Rotate and shift the arrow
-  PolygonRotateShift(Arrow, 5, sc[i],
+  PolygonRotateShift(Arrow, 4, sc[i],
                      traffic.track - (enable_north_up ?
                                              Angle::Zero() : heading));
 
@@ -422,7 +417,7 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
     canvas.Select(*target_brush);
 
   // Draw the polygon
-  canvas.DrawPolygon(Arrow, 5);
+  canvas.DrawPolygon(Arrow, 4);
 
   if (small) {
     if (!WarningMode() || traffic.HasAlarm())
@@ -438,7 +433,7 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
   // if vertical speed to small or negative -> skip this one
   if (side_display_type == SIDE_INFO_VARIO &&
       (!traffic.climb_rate_avg30s_available ||
-       traffic.climb_rate_avg30s < fixed(0.5) ||
+       traffic.climb_rate_avg30s < 0.5 ||
        traffic.IsPowered()))
       return;
 
@@ -455,19 +450,21 @@ FlarmTrafficWindow::PaintRadarTarget(Canvas &canvas,
     FormatRelativeUserAltitude(traffic.relative_altitude, tmp, true);
 
   PixelSize sz = canvas.CalcTextSize(tmp);
+  const PixelPoint tp{
+    sc[i].x + int(Layout::FastScale(11u)),
+    sc[i].y - int(sz.height / 2),
+  };
 
   // Draw vertical speed shadow
   canvas.SetTextColor(COLOR_WHITE);
-  canvas.DrawText(sc[i].x + Layout::FastScale(11) + 1,
-                  sc[i].y - sz.cy / 2 + 1, tmp);
-  canvas.DrawText(sc[i].x + Layout::FastScale(11) - 1,
-                  sc[i].y - sz.cy / 2 - 1, tmp);
+  canvas.DrawText({tp.x + 1, tp.y + 1}, tmp);
+  canvas.DrawText({tp.y - 1, tp.y - 1}, tmp);
 
   // Select color
   canvas.SetTextColor(*text_color);
 
   // Draw vertical speed
-  canvas.DrawText(sc[i].x + Layout::FastScale(11), sc[i].y - sz.cy / 2, tmp);
+  canvas.DrawText(tp, tmp);
 }
 
 void
@@ -484,7 +481,7 @@ FlarmTrafficWindow::PaintTargetInfoSmall(
 
   // Write the relativ altitude devided by 100 to the Buffer
   StaticString<10> buffer;
-  buffer.Format(_T("%d"), abs(relalt));
+  const auto relalt_s = buffer.Format(_T("%d"), abs(relalt));
 
   // Select font
   canvas.SetBackgroundTransparent();
@@ -492,19 +489,19 @@ FlarmTrafficWindow::PaintTargetInfoSmall(
   canvas.SetTextColor(text_color);
 
   // Calculate size of the output string
-  PixelSize tsize = canvas.CalcTextSize(buffer);
+  PixelSize tsize = canvas.CalcTextSize(relalt_s);
 
-  UPixelScalar dist = Layout::FastScale(traffic.HasAlarm() ? 12 : 8);
+  unsigned dist = Layout::FastScale(traffic.HasAlarm() ? 12 : 8);
 
   // Draw string
-  canvas.DrawText(sc[i].x + dist, sc[i].y - tsize.cy / 2, buffer);
+  canvas.DrawText(sc[i].At(dist, -int(tsize.height / 2)), relalt_s);
 
   // Set target_brush for the up/down arrow
   canvas.Select(arrow_brush);
   canvas.SelectNullPen();
 
   // Prepare the triangular polygon
-  RasterPoint triangle[4];
+  BulkPixelPoint triangle[4];
   triangle[0].x = 0;
   triangle[0].y = -4;
   triangle[1].x = 3;
@@ -522,8 +519,8 @@ FlarmTrafficWindow::PaintTargetInfoSmall(
     triangle[j].x = Layout::FastScale(triangle[j].x);
     triangle[j].y = Layout::FastScale(triangle[j].y);
 
-    triangle[j].x = sc[i].x + dist + triangle[j].x + tsize.cx / 2;
-    triangle[j].y = sc[i].y + flip * (triangle[j].y  - tsize.cy / 2);
+    triangle[j] = sc[i].At(dist + triangle[j].x + int(tsize.width / 2),
+                           flip * (triangle[j].y - int(tsize.height / 2)));
   }
   triangle[3].x = triangle[0].x;
   triangle[3].y = triangle[0].y;
@@ -582,17 +579,16 @@ FlarmTrafficWindow::PaintRadarPlane(Canvas &canvas) const
 {
   canvas.Select(look.plane_pen);
 
-  Point2D<int> p1(Layout::FastScale(small ? 5 : 10),
-                  -Layout::FastScale(small ? 1 : 2));
-  Point2D<int> p2(-p1.x, p1.y);
+  PixelPoint p1(Layout::FastScale(small ? 5 : 10),
+                -Layout::FastScale(small ? 1 : 2));
+  PixelPoint p2(-p1.x, p1.y);
 
   if (enable_north_up) {
     p1 = fir.Rotate(p1);
     p2 = fir.Rotate(p2);
   }
 
-  canvas.DrawLine(radar_mid.x + p1.x, radar_mid.y + p1.y,
-                  radar_mid.x + p2.x, radar_mid.y + p2.y);
+  canvas.DrawLine(radar_mid + p1, radar_mid + p2);
 
   p2 = { 0, Layout::FastScale(small ? 3 : 6) };
   p1 = { 0, -p2.y };
@@ -602,8 +598,7 @@ FlarmTrafficWindow::PaintRadarPlane(Canvas &canvas) const
     p2 = fir.Rotate(p2);
   }
 
-  canvas.DrawLine(radar_mid.x + p1.x, radar_mid.y + p1.y,
-                  radar_mid.x + p2.x, radar_mid.y + p2.y);
+  canvas.DrawLine(radar_mid + p1, radar_mid + p2);
 
   p1.x = Layout::FastScale(small ? 2 : 4);
   p1.y = p1.x;
@@ -614,8 +609,14 @@ FlarmTrafficWindow::PaintRadarPlane(Canvas &canvas) const
     p2 = fir.Rotate(p2);
   }
 
-  canvas.DrawLine(radar_mid.x + p1.x, radar_mid.y + p1.y,
-                  radar_mid.x + p2.x, radar_mid.y + p2.y);
+  canvas.DrawLine(radar_mid + p1, radar_mid + p2);
+}
+
+gcc_const
+static PixelPoint
+iround(DoublePoint2D p) noexcept
+{
+  return {iround(p.x), iround(p.y)};
 }
 
 /**
@@ -625,7 +626,7 @@ FlarmTrafficWindow::PaintRadarPlane(Canvas &canvas) const
 void
 FlarmTrafficWindow::PaintNorth(Canvas &canvas) const
 {
-  Point2D<fixed> p(fixed(0), fixed(-1));
+  DoublePoint2D p(0, -1);
   if (!enable_north_up) {
     p = fr.Rotate(p);
   }
@@ -636,11 +637,22 @@ FlarmTrafficWindow::PaintNorth(Canvas &canvas) const
   canvas.SetBackgroundTransparent();
   canvas.Select(look.label_font);
 
+  const PixelPoint q = radar_mid + iround(p * radius);
+
   PixelSize s = canvas.CalcTextSize(_T("N"));
-  canvas.DrawCircle(radar_mid.x + iround(p.x * radius),
-                radar_mid.y + iround(p.y * radius), s.cy * 0.65);
-  canvas.DrawText(radar_mid.x + iround(p.x * radius) - s.cx / 2,
-                  radar_mid.y + iround(p.y * radius) - s.cy / 2, _T("N"));
+  canvas.DrawCircle(q, s.height * 0.65);
+  canvas.DrawText(q - s / 2u, _T("N"));
+}
+
+static void
+DrawCircleLabel(Canvas &canvas, PixelPoint p,
+                BasicStringView<TCHAR> text) noexcept
+{
+  const auto size = canvas.CalcTextSize(text);
+  p.x -= size.width / 2;
+  p.y -= size.height * 3 / 4;
+
+  canvas.DrawText(p, text);
 }
 
 /**
@@ -655,8 +667,8 @@ FlarmTrafficWindow::PaintRadarBackground(Canvas &canvas) const
   canvas.SetTextColor(look.radar_color);
 
   // Paint circles
-  canvas.DrawCircle(radar_mid.x, radar_mid.y, radius);
-  canvas.DrawCircle(radar_mid.x, radar_mid.y, radius / 2);
+  canvas.DrawCircle(radar_mid, radius);
+  canvas.DrawCircle(radar_mid, radius / 2);
 
   PaintRadarPlane(canvas);
 
@@ -670,16 +682,13 @@ FlarmTrafficWindow::PaintRadarBackground(Canvas &canvas) const
 
   TCHAR distance_string[10];
   FormatUserDistanceSmart(distance, distance_string,
-                          ARRAY_SIZE(distance_string), fixed(1000));
-  PixelSize s = canvas.CalcTextSize(distance_string);
-  canvas.DrawText(radar_mid.x - s.cx / 2,
-                  radar_mid.y + radius - s.cy * 0.75, distance_string);
+                          ARRAY_SIZE(distance_string), 1000);
+  DrawCircleLabel(canvas, radar_mid + PixelSize{0u, radius}, distance_string);
 
   FormatUserDistanceSmart(distance / 2, distance_string,
-                          ARRAY_SIZE(distance_string), fixed(1000));
-  s = canvas.CalcTextSize(distance_string);
-  canvas.DrawText(radar_mid.x - s.cx / 2,
-                  radar_mid.y + radius / 2 - s.cy * 0.75, distance_string);
+                          ARRAY_SIZE(distance_string), 1000);
+  DrawCircleLabel(canvas, radar_mid + PixelSize{0u, radius / 2},
+                  distance_string);
 
   canvas.SetBackgroundTransparent();
 
@@ -712,11 +721,11 @@ FlarmTrafficWindow::OnPaint(Canvas &canvas)
 {
 #ifdef ENABLE_OPENGL
   if (small) {
-    const GLBlend blend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    const ScopeAlphaBlend alpha_blend;
 
     canvas.SelectBlackPen();
     canvas.Select(Brush(look.background_color.WithAlpha(0xd0)));
-    canvas.DrawCircle(radar_mid.x, radar_mid.y, radius);
+    canvas.DrawCircle(radar_mid, radius);
 
   } else
 #endif
@@ -726,7 +735,7 @@ FlarmTrafficWindow::OnPaint(Canvas &canvas)
 }
 
 bool
-FlarmTrafficWindow::SelectNearTarget(int x, int y, int max_distance)
+FlarmTrafficWindow::SelectNearTarget(PixelPoint p, int max_distance)
 {
   int min_distance = 99999;
   int min_id = -1;
@@ -736,8 +745,7 @@ FlarmTrafficWindow::SelectNearTarget(int x, int y, int max_distance)
     if (!data.list[i].IsDefined())
       continue;
 
-    int distance_sq = (x - sc[i].x) * (x - sc[i].x) +
-                      (y - sc[i].y) * (y - sc[i].y);
+    int distance_sq = (p - sc[i]).MagnitudeSquared();
 
     if (distance_sq > min_distance
         || distance_sq > max_distance * max_distance)
