@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,7 +25,7 @@ Copyright_License {
 
 #ifdef HAVE_CPU_FREQUENCY
 
-#include "OS/FileUtil.hpp"
+#include "system/FileUtil.hpp"
 
 #include <atomic>
 
@@ -33,7 +33,7 @@ static bool
 SetCPUFrequencyGovernor(const char *governor)
 {
 #ifdef __linux__
-  return File::WriteExisting("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor",
+  return File::WriteExisting(Path("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor"),
                              governor);
 #else
   return false;
