@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -35,12 +35,20 @@ JobThread::Start()
 }
 
 void
-JobThread::Run()
+JobThread::Run() noexcept
 {
   assert(!running.load(std::memory_order_relaxed));
 
   running.store(true, std::memory_order_relaxed);
-  job.Run(*this);
+
+  try {
+    job.Run(*this);
+  } catch (...) {
+    /* an exception was thrown by the Job: remember it, rethrow it in
+       the calling thread in Join() */
+    exception = std::current_exception();
+  }
+
   running.store(false, std::memory_order_relaxed);
 
   SendNotification();
@@ -56,3 +64,14 @@ JobThread::OnNotification()
     was_running = false;
   }
 }
+
+void
+JobThread::Join()
+{
+  Thread::Join();
+
+  if (exception)
+    /* rethrow the exception that was thrown by Job::Run() in the
+       calling thread */
+    std::rethrow_exception(std::move(exception));
+}
