@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,10 +24,13 @@ Copyright_License {
 #ifndef XCSOAR_LOCAL_PATH_HPP
 #define XCSOAR_LOCAL_PATH_HPP
 
-#include "Compiler.h"
+#include "util/Compiler.h"
 
 #include <tchar.h>
 
+class Path;
+class AllocatedPath;
+
 namespace File {
   class Visitor;
 }
@@ -56,25 +59,32 @@ CreateDataPath();
  * Overrides the detected primary data path.
  */
 void
-SetPrimaryDataPath(const TCHAR *path);
+SetPrimaryDataPath(Path path);
 
 /**
  * Returns the absolute path of the primary data directory.
  */
-const TCHAR *
+Path
 GetPrimaryDataPath();
 
 /**
- * Returns the path of the XCSoarData folder, optionally including
- * the given file name
- * @param buffer Output buffer
- * @param file optional filename to include in the output
+ * Gives the position of an XCSoar data file within the particular file
+ * system of this host.
+ * @param file The name of the file in question. Should not be equal to 
+ *             'nullptr'.
+ * @return The fully qualified path of file.
  */
-const TCHAR *
-LocalPath(TCHAR* buf, const TCHAR *file);
+AllocatedPath
+LocalPath(Path file);
 
-const TCHAR *
-LocalPath(TCHAR *buffer, const TCHAR *subdir, const TCHAR *name);
+AllocatedPath
+LocalPath(const TCHAR *file);
+
+/**
+ * Create a subdirectory of XCSoarData and return its absolute path.
+ */
+AllocatedPath
+MakeLocalPath(const TCHAR *name);
 
 /**
  * Return the portion of the specified path that is relative to the
@@ -82,23 +92,27 @@ LocalPath(TCHAR *buffer, const TCHAR *subdir, const TCHAR *name);
  * inside the primary data path).
  */
 gcc_pure
-const TCHAR *
-RelativePath(const TCHAR *path);
+Path
+RelativePath(Path path);
 
 /**
  * Converts a file path by replacing %LOCAL_PATH% with the full pathname to
  * the XCSoarData folder
  * @param filein Pointer to the string to convert
  */
-void
-ExpandLocalPath(TCHAR *dest, const TCHAR *src);
+gcc_pure
+AllocatedPath
+ExpandLocalPath(Path src);
 
 /**
  * Converts a file path from full pathname to a shorter version with the
  * XCSoarData folder replaced by %LOCAL_PATH%
  * @param filein Pointer to the string to convert
+ * @return the new path or nullptr if the given path cannot be contracted
  */
-void ContractLocalPath(TCHAR* filein);
+gcc_pure
+AllocatedPath
+ContractLocalPath(Path src);
 
 void VisitDataFiles(const TCHAR* filter, File::Visitor &visitor);
 
