@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -21,46 +21,45 @@
  */
 
 #include "SelfTimingKalmanFilter1d.hpp"
-#include "OS/Clock.hpp"
+#include "system/Clock.hpp"
 
 #include <algorithm>
 
-SelfTimingKalmanFilter1d::SelfTimingKalmanFilter1d(const fixed max_dt,
-                                                   const fixed var_x_accel)
-    : filter_(var_x_accel), t_last_update_ms_(0) {
+SelfTimingKalmanFilter1d::SelfTimingKalmanFilter1d(const double max_dt,
+                                                   const double var_x_accel)
+    : filter_(var_x_accel) {
   SetMaxDt(max_dt);
 }
 
-SelfTimingKalmanFilter1d::SelfTimingKalmanFilter1d(const fixed max_dt)
-    : t_last_update_ms_(0) {
+SelfTimingKalmanFilter1d::SelfTimingKalmanFilter1d(const double max_dt) {
   SetMaxDt(max_dt);
 }
 
 void
-SelfTimingKalmanFilter1d::SetMaxDt(const fixed max_dt)
+SelfTimingKalmanFilter1d::SetMaxDt(const double max_dt)
 {
   // It's OK, albeit silly, to have a zero max_dt value. We just always reset.
-  max_dt_ms_ = negative(max_dt) ? 0U : (unsigned int)(max_dt * 1000);
+  max_dt_us_ = max_dt < 0 ? 0u : unsigned(max_dt * 1e6);
 }
 
-fixed
+double
 SelfTimingKalmanFilter1d::GetMaxDt() const
 {
-  return fixed(max_dt_ms_) / 1000;
+  return max_dt_us_ / 1e6;
 }
 
 void
-SelfTimingKalmanFilter1d::Update(const fixed z_abs, const fixed var_z_abs)
+SelfTimingKalmanFilter1d::Update(const double z_abs, const double var_z_abs)
 {
-  const unsigned int t_ms = MonotonicClockMS();
+  const unsigned int t_us = MonotonicClockUS();
 
-  /* if we're called too quickly (less than 1ms), round dt up to 1ms
+  /* if we're called too quickly (less than 1us), round dt up to 1us
      to avoid problems in KalmanFilter1d::Update() */
-  const unsigned int dt_ms = std::max(t_ms - t_last_update_ms_, 1u);
+  const unsigned int dt_us = std::max(t_us - t_last_update_us_, uint64_t(1));
 
-  t_last_update_ms_ = t_ms;
+  t_last_update_us_ = t_us;
 
-  if (dt_ms > max_dt_ms_)
+  if (dt_us > max_dt_us_)
     filter_.Reset();
-  filter_.Update(z_abs, var_z_abs, fixed(dt_ms) / 1000);
+  filter_.Update(z_abs, var_z_abs, dt_us / 1e6);
 }
