@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@ namespace ProfileKeys {
 
 extern const char ShowMenuButton[];
 extern const char UIScale[];
+extern const char CustomDPI[];
 extern const char Password[];
 extern const char SpeedUnitsValue[];
 extern const char DistanceUnitsValue[];
@@ -61,6 +62,7 @@ extern const char AdditionalWaypointFile[];
 extern const char WatchedWaypointFile[];
 extern const char AirspaceFile[];
 extern const char AdditionalAirspaceFile[];
+extern const char FlarmFile[];
 extern const char AirfieldFile[];
 extern const char PolarFile[];
 extern const char LanguageFile[];
@@ -92,6 +94,7 @@ extern const char HomeWaypoint[];
 extern const char HomeLocation[];
 extern const char TeamcodeRefWaypoint[];
 extern const char PilotName[];
+extern const char CoPilotName[];
 extern const char AircraftType[];
 extern const char AircraftReg[];
 extern const char CompetitionId[];
@@ -103,7 +106,6 @@ extern const char SoundAudioVario[];
 extern const char SoundTask[];
 extern const char SoundModes[];
 extern const char NettoSpeed[];
-extern const char AutoBlank[];
 extern const char AverEffTime[]; // VENTA6
 extern const char VarioGauge[];
 extern const char AppIndLandable[];
@@ -124,8 +126,10 @@ extern const char HapticFeedback[];
 extern const char AppDialogTabStyle[];
 extern const char AppDialogStyle[];
 extern const char AppInfoBoxBorder[];
-extern const char AppInfoBoxModel[]; // VENTA-ADDON MODEL CHANGE
 extern const char AppAveNeedle[];
+extern const char AppAveThermalNeedle[];
+extern const char CursorSize[];
+extern const char CursorColorsInverted[];
 extern const char AutoAdvance[];
 extern const char UTCOffset[];
 extern const char UTCOffsetSigned[];
@@ -138,6 +142,7 @@ extern const char TerrainRamp[];
 extern const char EnableFLARMMap[];
 extern const char EnableFLARMGauge[];
 extern const char AutoCloseFlarmDialog[];
+extern const char TAPosition[];
 extern const char EnableTAGauge[];
 extern const char EnableThermalProfile[];
 extern const char TrailDrift[];
@@ -146,15 +151,6 @@ extern const char DisplayTrackBearing[];
 extern const char GliderScreenPosition[];
 extern const char SetSystemTimeFromGPS[];
 
-extern const char VoiceClimbRate[];
-extern const char VoiceTerrain[];
-extern const char VoiceWaypointDistance[];
-extern const char VoiceTaskAltitudeDifference[];
-extern const char VoiceMacCready[];
-extern const char VoiceNewWaypoint[];
-extern const char VoiceInSector[];
-extern const char VoiceAirspace[];
-
 extern const char FinishMinHeight[];
 extern const char FinishHeightRef[];
 extern const char StartMaxHeight[];
@@ -182,7 +178,9 @@ extern const char AbortTaskMode[];
 extern const char AutoMcMode[];
 extern const char AutoMc[];
 extern const char EnableExternalTriggerCruise[];
-extern const char OLCRules[];
+extern const char CruiseToCirclingModeSwitchThreshold[];
+extern const char CirclingToCruiseModeSwitchThreshold[];
+extern const char OLCRules[]; // legacy name, key contains contest rules
 extern const char PredictContest[];
 extern const char Handicap[];
 extern const char SnailWidthScale[];
@@ -242,23 +240,30 @@ extern const char AircraftSymbol[];
 
 extern const char FlarmLocation[];
 
-extern const char TrackingInterval[];
-extern const char TrackingVehicleType[];
-extern const char TrackingVehicleName[];
 extern const char SkyLinesTrackingEnabled[];
 extern const char SkyLinesRoaming[];
 extern const char SkyLinesTrackingInterval[];
 extern const char SkyLinesTrafficEnabled[];
 extern const char SkyLinesNearTrafficEnabled[];
+extern const char SkyLinesTrafficMapMode[];
 extern const char SkyLinesTrackingKey[];
 
 extern const char CloudEnabled[];
+extern const char CloudShowThermals[];
 extern const char CloudKey[];
 
 extern const char LiveTrack24Enabled[];
 extern const char LiveTrack24Server[];
 extern const char LiveTrack24Username[];
 extern const char LiveTrack24Password[];
+extern const char LiveTrack24TrackingInterval[];
+extern const char LiveTrack24TrackingVehicleType[];
+extern const char LiveTrack24TrackingVehicleName[];
+
+extern const char PCMetUsername[];
+extern const char PCMetPassword[];
+extern const char PCMetFtpUsername[];
+extern const char PCMetFtpPassword[];
 
 extern const char EnableLocationMapItem[];
 extern const char EnableArrivalAltitudeMapItem[];
@@ -276,6 +281,8 @@ extern const char PagesDistinctZoom[];
 
 extern const char WaveAssistant[];
 
+extern const char MasterAudioVolume[];
+
 }
 
 #endif
