@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,33 +23,32 @@ Copyright_License {
 
 #include "RateLimiter.hpp"
 
-#include <assert.h>
+#include <cassert>
 
-RateLimiter::RateLimiter(unsigned _period_ms, unsigned _delay_ms)
-  :period_ms(_period_ms - _delay_ms), delay_ms(_delay_ms)
+RateLimiter::RateLimiter(std::chrono::steady_clock::duration _period,
+                         std::chrono::steady_clock::duration _delay) noexcept
+  :period(_period - _delay), delay(_delay)
 {
-  assert(_period_ms >= _delay_ms);
+  assert(_period >= _delay);
 }
 
 void
 RateLimiter::Trigger()
 {
-  if (IsActive())
+  if (timer.IsPending())
     return;
 
-  unsigned schedule_ms = delay_ms;
-  int elapsed = clock.Elapsed();
-  if (elapsed >= 0 && (unsigned)elapsed < period_ms)
-    schedule_ms += period_ms - elapsed;
+  std::chrono::steady_clock::duration schedule = delay;
+  const auto elapsed = clock.Elapsed();
+  if (elapsed.count() >= 0 && elapsed < period)
+    schedule += period - elapsed;
 
-  Schedule(schedule_ms);
+  timer.Schedule(schedule);
 }
 
 void
 RateLimiter::OnTimer()
 {
-  Timer::Cancel();
-
   clock.Update();
   Run();
 }
