@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,22 +24,24 @@ Copyright_License {
 #ifndef XCSOAR_WAYPOINT_LABEL_LIST_HPP
 #define XCSOAR_WAYPOINT_LABEL_LIST_HPP
 
-#include "Rough/RoughAltitude.hpp"
 #include "Renderer/TextInBox.hpp"
-#include "Screen/Point.hpp"
-#include "Util/NonCopyable.hpp"
-#include "Util/StaticArray.hpp"
+#include "ui/dim/Point.hpp"
+#include "ui/dim/Rect.hpp"
+#include "util/NonCopyable.hpp"
+#include "util/StaticArray.hxx"
 #include "Sizes.h" /* for NAME_SIZE */
 
 #include <tchar.h>
 
 class WaypointLabelList : private NonCopyable {
+  static constexpr int WPCIRCLESIZE = 2;
+
 public:
   struct Label{
     TCHAR Name[NAME_SIZE+1];
-    RasterPoint Pos;
+    PixelPoint Pos;
     TextInBoxMode Mode;
-    RoughAltitude AltArivalAGL;
+    int AltArivalAGL;
     bool inTask;
     bool isLandable;
     bool isAirport;
@@ -48,33 +50,32 @@ public:
   };
 
 protected:
-  UPixelScalar width, height;
-  PixelRect bounds;
+  PixelRect clip_rect;
 
   StaticArray<Label, 256u> labels;
 
 public:
-  WaypointLabelList(UPixelScalar _width, UPixelScalar _height)
-    :width(_width), height(_height) {}
+  explicit WaypointLabelList(PixelRect _rect) noexcept
+    :clip_rect(_rect)
+  {
+    clip_rect.Grow(WPCIRCLESIZE);
+    clip_rect.right += WPCIRCLESIZE * 2;
+  }
 
-  void Add(const TCHAR *Name, PixelScalar X, PixelScalar Y,
+  void Add(const TCHAR *name, PixelPoint p,
            TextInBoxMode Mode, bool bold,
-           RoughAltitude AltArivalAGL,
+           int AltArivalAGL,
            bool inTask, bool isLandable, bool isAirport,
-           bool isWatchedWaypoint);
-  void Sort();
+           bool isWatchedWaypoint) noexcept;
+  void Sort() noexcept;
 
-  const Label *begin() const {
+  const Label *begin() const noexcept {
     return labels.begin();
   }
 
-  const Label *end() const {
+  const Label *end() const noexcept {
     return labels.end();
   }
-
-  const Label &operator[](unsigned i) const {
-    return labels[i];
-  }
 };
 
 #endif
