@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@ Copyright_License {
 #include "Simulator.hpp"
 #include "InfoBoxes/InfoBoxManager.hpp"
 #include "Terrain/RasterTerrain.hpp"
-#include "Terrain/RasterWeatherStore.hpp"
+#include "Weather/Rasp/RaspStore.hpp"
 #include "Input/InputEvents.hpp"
 #include "Input/InputQueue.hpp"
 #include "Dialogs/StartupDialog.hpp"
@@ -52,8 +52,10 @@ Copyright_License {
 #include "Device/MultipleDevices.hpp"
 #include "Topography/TopographyStore.hpp"
 #include "Topography/TopographyGlue.hpp"
+#include "Audio/Features.hpp"
+#include "Audio/GlobalVolumeController.hpp"
 #include "Audio/VarioGlue.hpp"
-#include "Screen/Busy.hpp"
+#include "Audio/VolumeController.hpp"
 #include "CommandLine.hpp"
 #include "MainWindow.hpp"
 #include "Computer/GlideComputer.hpp"
@@ -64,12 +66,14 @@ Copyright_License {
 #include "CalculationThread.hpp"
 #include "Replay/Replay.hpp"
 #include "LocalPath.hpp"
-#include "IO/FileCache.hpp"
-#include "Net/HTTP/DownloadManager.hpp"
-#include "Hardware/AltairControl.hpp"
+#include "io/FileCache.hpp"
+#include "io/async/AsioThread.hpp"
+#include "io/async/GlobalAsioThread.hpp"
+#include "net/http/Init.hpp"
+#include "net/http/DownloadManager.hpp"
 #include "Hardware/DisplayDPI.hpp"
 #include "Hardware/DisplayGlue.hpp"
-#include "Compiler.h"
+#include "util/Compiler.h"
 #include "NMEA/Aircraft.hpp"
 #include "Waypoint/Waypoints.hpp"
 #include "Waypoint/WaypointGlue.hpp"
@@ -93,15 +97,23 @@ Copyright_License {
 #include "Tracking/TrackingGlue.hpp"
 #include "Units/Units.hpp"
 #include "Formatter/UserGeoPointFormatter.hpp"
-#include "Thread/Debug.hpp"
+#include "thread/Debug.hpp"
+
+#include "lua/StartFile.hpp"
+#include "lua/Background.hpp"
 
 #ifdef ENABLE_OPENGL
-#include "Screen/OpenGL/Globals.hpp"
-#include "Screen/OpenGL/Dynamic.hpp"
+#include "ui/canvas/opengl/Globals.hpp"
+#include "ui/canvas/opengl/Dynamic.hpp"
 #else
 #include "DrawThread.hpp"
 #endif
 
+#ifdef ANDROID
+#include "Android/Main.hpp"
+#include "Android/Context.hpp"
+#endif
+
 static TaskManager *task_manager;
 static GlideComputerEvents *glide_computer_events;
 static AllMonitors *all_monitors;
@@ -110,7 +122,7 @@ static GlideComputerTaskEvents *task_events;
 static bool
 LoadProfile()
 {
-  if (StringIsEmpty(Profile::GetPath()) &&
+  if (Profile::GetPath().IsNull() &&
       !dlgStartupShowModal())
     return false;
 
@@ -120,10 +132,6 @@ LoadProfile()
   Units::SetConfig(CommonInterface::GetUISettings().format.units);
   SetUserCoordinateFormat(CommonInterface::GetUISettings().format.coordinate_format);
 
-#ifdef HAVE_MODEL_TYPE
-  global_model_type = CommonInterface::GetSystemSettings().model_type;
-#endif
-
   return true;
 }
 
@@ -132,14 +140,21 @@ AfterStartup()
 {
   StartupLogFreeRamAndStorage();
 
+  try {
+    const auto lua_path = LocalPath(_T("lua"));
+    Lua::StartFile(AllocatedPath::Build(lua_path, _T("init.lua")));
+  } catch (...) {
+      LogError(std::current_exception());
+  }
+
   if (is_simulator()) {
     InputEvents::processGlideComputer(GCE_STARTUP_SIMULATOR);
   } else {
     InputEvents::processGlideComputer(GCE_STARTUP_REAL);
   }
 
-  OrderedTask *defaultTask = LoadDefaultTask(CommonInterface::GetComputerSettings().task,
-                                             &way_points);
+  const auto defaultTask = LoadDefaultTask(CommonInterface::GetComputerSettings().task,
+                                           &way_points);
   if (defaultTask) {
     {
       ScopeSuspendAllThreads suspend;
@@ -148,15 +163,10 @@ AfterStartup()
     }
 
     protected_task_manager->TaskCommit(*defaultTask);
-    delete defaultTask;
   }
 
   task_manager->Resume();
 
-#ifdef USE_GDI
-  CommonInterface::main_window->Fullscreen();
-#endif
-
   InfoBoxManager::SetDirty();
 
   ForceCalculation();
@@ -173,24 +183,21 @@ Startup()
 {
   VerboseOperationEnvironment operation;
 
-#ifdef USE_GDI
-  //If "XCSoar" is already running, stop this instance
-  if (MainWindow::Find())
-    return false;
-#endif
-
 #ifdef HAVE_DOWNLOAD_MANAGER
   Net::DownloadManager::Initialise();
 #endif
 
   // Creates the main window
 
-  TopWindowStyle style;
+  UI::TopWindowStyle style;
   if (CommandLine::full_screen)
     style.FullScreen();
 
-  if (!IsWindowsCE())
-    style.Resizable();
+  style.Resizable();
+
+#ifdef SOFTWARE_ROTATE_DISPLAY
+  style.InitialOrientation(Display::DetectInitialOrientation());
+#endif
 
   MainWindow *const main_window = CommonInterface::main_window =
     new MainWindow();
@@ -202,28 +209,14 @@ Startup()
 
 #ifdef ENABLE_OPENGL
   LogFormat("OpenGL: "
-#ifdef HAVE_DYNAMIC_EGL
-            "egl=%d "
-#endif
-#ifdef HAVE_OES_DRAW_TEXTURE
-            "oesdt=%d "
-#endif
 #ifdef HAVE_DYNAMIC_MULTI_DRAW_ARRAYS
             "mda=%d "
 #endif
-            "npot=%d vbo=%d fbo=%d stencil=%#x",
-#ifdef HAVE_DYNAMIC_EGL
-             OpenGL::egl,
-#endif
-#ifdef HAVE_OES_DRAW_TEXTURE
-            OpenGL::oes_draw_texture,
-#endif
+            "npot=%d stencil=%#x",
 #ifdef HAVE_DYNAMIC_MULTI_DRAW_ARRAYS
             GLExt::HaveMultiDrawElements(),
 #endif
              OpenGL::texture_non_power_of_two,
-             OpenGL::vertex_buffer_object,
-            OpenGL::frame_buffer_object,
             OpenGL::render_buffer_stencil);
 #endif
 
@@ -271,8 +264,16 @@ Startup()
   main_window->InitialiseConfigured();
 
   {
-    TCHAR buffer[MAX_PATH];
-    file_cache = new FileCache(LocalPath(buffer, _T("cache")));
+#ifdef ANDROID
+    auto cache_path = context->GetExternalCacheDir(Java::GetEnv());
+    if (cache_path == nullptr)
+      throw std::runtime_error("No Android cache directory");
+
+    // TODO: delete the old cache directory in XCSoarData?
+#else
+    auto cache_path = LocalPath(_T("cache"));
+#endif
+    file_cache = new FileCache(std::move(cache_path));
   }
 
   ReadLanguageFile();
@@ -281,24 +282,9 @@ Startup()
 
   // Initialize DeviceBlackboard
   device_blackboard = new DeviceBlackboard();
-  devices = new MultipleDevices();
+  devices = new MultipleDevices(*asio_thread, *global_cares_channel);
   device_blackboard->SetDevices(*devices);
 
-#ifdef HAVE_AYGSHELL_DLL
-  const AYGShellDLL &ayg = main_window->ayg_shell_dll;
-  ayg.SHSetAppKeyWndAssoc(VK_APP1, *main_window);
-  ayg.SHSetAppKeyWndAssoc(VK_APP2, *main_window);
-  ayg.SHSetAppKeyWndAssoc(VK_APP3, *main_window);
-  ayg.SHSetAppKeyWndAssoc(VK_APP4, *main_window);
-  // Typical Record Button
-  //	Why you can't always get this to work
-  //	http://forums.devbuzz.com/m_1185/mpage_1/key_/tm.htm
-  //	To do with the fact it is a global hotkey, but you can with code above
-  //	Also APPA is record key on some systems
-  ayg.SHSetAppKeyWndAssoc(VK_APP5, *main_window);
-  ayg.SHSetAppKeyWndAssoc(VK_APP6, *main_window);
-#endif
-
   // Initialize main blackboard data
   task_events = new GlideComputerTaskEvents();
   task_manager = new TaskManager(computer_settings.task, way_points);
@@ -315,10 +301,10 @@ Startup()
 
   logger = new Logger();
 
-  glide_computer = new GlideComputer(way_points, airspace_database,
+  glide_computer = new GlideComputer(computer_settings,
+                                     way_points, airspace_database,
                                      *protected_task_manager,
                                      *task_events);
-  glide_computer->ReadComputerSettings(computer_settings);
   glide_computer->SetTerrain(terrain);
   glide_computer->SetLogger(logger);
   glide_computer->Initialise();
@@ -326,13 +312,18 @@ Startup()
   replay = new Replay(logger, *protected_task_manager);
 
 #ifdef HAVE_CMDLINE_REPLAY
-  if (CommandLine::replay_path != nullptr)
-    replay->Start(CommandLine::replay_path);
+  if (CommandLine::replay_path != nullptr) {
+    try {
+      replay->Start(Path(CommandLine::replay_path));
+    } catch (...) {
+      LogError(std::current_exception());
+    }
+  }
 #endif
 
 
   GlidePolar &gp = CommonInterface::SetComputerSettings().polar.glide_polar_task;
-  gp = GlidePolar(fixed(0));
+  gp = GlidePolar(0);
   gp.SetMC(computer_settings.task.safety_mc);
   gp.SetBugs(computer_settings.polar.degradation_factor);
   PlaneGlue::FromProfile(CommonInterface::SetComputerSettings().plane,
@@ -363,8 +354,8 @@ Startup()
 
   // Scan for weather forecast
   LogFormat("RASP load");
-  rasp = new RasterWeatherStore();
-  rasp->ScanAll(CommonInterface::Basic().location, operation);
+  auto rasp = std::make_shared<RaspStore>(LocalPath(_T(RASP_FILENAME)));
+  rasp->ScanAll();
 
   // Reads the airspace files
   ReadAirspace(airspace_database, terrain, computer_settings.pressure,
@@ -383,6 +374,10 @@ Startup()
   noaa_store->LoadFromProfile();
 #endif
 
+#ifdef HAVE_VOLUME_CONTROLLER
+  volume_controller->SetVolume(ui_settings.sound.master_volume);
+#endif
+
   AudioVarioGlue::Initialise();
   AudioVarioGlue::Configure(ui_settings.sound.vario);
 
@@ -409,7 +404,7 @@ Startup()
 
     map_window->SetTopography(topography);
     map_window->SetTerrain(terrain);
-    map_window->SetWeather(rasp);
+    map_window->SetRasp(rasp);
 
 #ifdef HAVE_NOAA
     map_window->SetNOAAStore(noaa_store);
@@ -445,8 +440,7 @@ Startup()
 
   if (!is_simulator() && computer_settings.logger.enable_flight_logger) {
     flight_logger = new GlueFlightLogger(live_blackboard);
-    TCHAR buffer[MAX_PATH];
-    flight_logger->SetPath(LocalPath(buffer, _T("flights.log")));
+    flight_logger->SetPath(LocalPath(_T("flights.log")));
   }
 
   if (computer_settings.logger.enable_nmea_logger)
@@ -464,10 +458,10 @@ Startup()
   PageActions::Update();
 
 #ifdef HAVE_TRACKING
-  tracking = new TrackingGlue();
+  tracking = new TrackingGlue(*asio_thread, *Net::curl);
   tracking->SetSettings(computer_settings.tracking);
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
   if (map_window != nullptr)
     map_window->SetSkyLinesData(&tracking->GetSkyLinesData());
 #endif
@@ -489,7 +483,6 @@ void
 Shutdown()
 {
   VerboseOperationEnvironment operation;
-  gcc_unused ScopeBusyIndicator busy;
 
   MainWindow *const main_window = CommonInterface::main_window;
   auto &live_blackboard = CommonInterface::GetLiveBlackboard();
@@ -504,6 +497,8 @@ Shutdown()
 
   StartupLogFreeRamAndStorage();
 
+  Lua::StopAllBackground();
+
   // Turn off all displays
   global_running = false;
 
@@ -589,18 +584,19 @@ Shutdown()
   operation.SetText(_("Shutdown, saving task..."));
 
   LogFormat("Save default task");
-  if (protected_task_manager != nullptr)
-    protected_task_manager->TaskSaveDefault();
+  if (protected_task_manager != nullptr) {
+    try {
+      protected_task_manager->TaskSaveDefault();
+    } catch (...) {
+      LogError(std::current_exception());
+    }
+  }
 
   // Clear waypoint database
   way_points.Clear();
 
   operation.SetText(_("Shutdown, please wait..."));
 
-  // Clear weather database
-  delete rasp;
-  rasp = nullptr;
-
   // Clear terrain database
 
   delete terrain;
