@@ -27,9 +27,9 @@
  */
 
 #include "Node.hpp"
-#include "Util/StringAPI.hxx"
+#include "util/StringAPI.hxx"
 
-#include <assert.h>
+#include <cassert>
 
 XMLNode
 XMLNode::CreateRoot(const TCHAR *name)
