@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@ Copyright_License {
 #include "Main.hpp"
 #include "MapWindow/MapWindow.hpp"
 #include "Terrain/RasterTerrain.hpp"
+#include "Terrain/Loader.hpp"
 #include "Profile/ProfileKeys.hpp"
 #include "Profile/ComputerProfile.hpp"
 #include "Profile/MapProfile.hpp"
@@ -42,13 +43,13 @@ Copyright_License {
 #include "Engine/Waypoint/Waypoints.hpp"
 #include "Engine/Airspace/Airspaces.hpp"
 #include "LogFile.hpp"
-#include "IO/ConfiguredFile.hpp"
-#include "IO/LineReader.hpp"
+#include "io/ConfiguredFile.hpp"
+#include "io/LineReader.hpp"
 #include "Operation/Operation.hpp"
-#include "Thread/Debug.hpp"
+#include "thread/Debug.hpp"
 
 void
-DeviceBlackboard::SetStartupLocation(const GeoPoint &loc, const fixed alt) {}
+DeviceBlackboard::SetStartupLocation(const GeoPoint &loc, const double alt) {}
 
 #ifndef NDEBUG
 
@@ -123,11 +124,10 @@ LoadFiles(PlacesOfInterestSettings &poi_settings,
   WaypointGlue::SetHome(way_points, terrain, poi_settings, team_code_settings,
                         NULL, false);
 
-  std::unique_ptr<TLineReader> reader(OpenConfiguredTextFile(ProfileKeys::AirspaceFile,
-                                                             Charset::AUTO));
+  auto reader = OpenConfiguredTextFile(ProfileKeys::AirspaceFile,
+                                       Charset::AUTO);
   if (reader) {
-    AirspaceParser parser(airspace_database);
-    parser.Parse(*reader, operation);
+    ParseAirspaceFile(airspace_database, *reader, operation);
     airspace_database.Optimise();
   }
 }
@@ -140,8 +140,8 @@ GenerateBlackboard(MapWindow &map, const ComputerSettings &settings_computer,
   DerivedInfo derived_info;
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
-  nmea_info.time = fixed(1297230000);
+  nmea_info.clock = 1;
+  nmea_info.time = 1297230000;
   nmea_info.alive.Update(nmea_info.clock);
 
   if (settings_computer.poi.home_location_available)
@@ -154,20 +154,16 @@ GenerateBlackboard(MapWindow &map, const ComputerSettings &settings_computer,
   nmea_info.location_available.Update(nmea_info.clock);
   nmea_info.track = Angle::Degrees(90);
   nmea_info.track_available.Update(nmea_info.clock);
-  nmea_info.ground_speed = fixed(50);
+  nmea_info.ground_speed = 50;
   nmea_info.ground_speed_available.Update(nmea_info.clock);
-  nmea_info.gps_altitude = fixed(1500);
+  nmea_info.gps_altitude = 1500;
   nmea_info.gps_altitude_available.Update(nmea_info.clock);
 
   derived_info.Reset();
   derived_info.terrain_valid = true;
 
-  if (terrain != NULL) {
-    RasterTerrain::UnprotectedLease lease(*terrain);
-    do {
-      lease->SetViewCenter(nmea_info.location, fixed(50000));
-    } while (lease->IsDirty());
-  }
+  if (terrain != nullptr)
+    while (terrain->UpdateTiles(nmea_info.location, 50000)) {}
 
   map.ReadBlackboard(nmea_info, derived_info, settings_computer,
                      settings_map);
