@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,17 +24,21 @@
 #include "harness_airspace.hpp"
 #include "Route/AirspaceRoute.hpp"
 #include "Engine/Airspace/AirspaceAircraftPerformance.hpp"
+#include "Engine/Airspace/Predicate/AirspacePredicate.hpp"
 #include "Geo/SpeedVector.hpp"
 #include "Geo/GeoVector.hpp"
 #include "GlideSolvers/GlideSettings.hpp"
 #include "GlideSolvers/GlidePolar.hpp"
 #include "Terrain/RasterMap.hpp"
-#include "OS/ConvertPathName.hpp"
-#include "OS/FileUtil.hpp"
+#include "Terrain/Loader.hpp"
+#include "system/ConvertPathName.hpp"
+#include "system/FileUtil.hpp"
 #include "Compatibility/path.h"
 #include "Operation/Operation.hpp"
 #include "test_debug.hpp"
 
+#include <zzip/zzip.h>
+
 #include <fstream>
 
 #include <string.h>
@@ -52,7 +56,7 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
   setup_airspaces(airspaces, map.GetMapCenter(), n_airspaces);
 
   {
-    Directory::Create(_T("output/results"));
+    Directory::Create(Path(_T("output/results")));
     std::ofstream fout("output/results/terrain.txt");
 
     unsigned nx = 100;
@@ -61,13 +65,13 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
 
     for (unsigned i = 0; i < nx; ++i) {
       for (unsigned j = 0; j < ny; ++j) {
-        fixed fx = (fixed)i / (nx - 1) * 2 - fixed(1);
-        fixed fy = (fixed)j / (ny - 1) * 2 - fixed(1);
-        GeoPoint x(origin.longitude + Angle::Degrees(fixed(0.2) + fixed(0.7) * fx),
-                   origin.latitude + Angle::Degrees(fixed(0.9) * fy));
-        short h = map.GetInterpolatedHeight(x);
+        auto fx = (double)i / (nx - 1) * 2 - 1;
+        auto fy = (double)j / (ny - 1) * 2 - 1;
+        GeoPoint x(origin.longitude + Angle::Degrees(0.2 + 0.7 * fx),
+                   origin.latitude + Angle::Degrees(0.9 * fy));
+        auto h = map.GetInterpolatedHeight(x);
         fout << x.longitude.Degrees() << " " << x.latitude.Degrees()
-             << " " << h << "\n";
+             << " " << h.GetValue() << "\n";
       }
 
       fout << "\n";
@@ -84,15 +88,15 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
     GeoPoint p_dest(Angle::Degrees(0.8), Angle::Degrees(-0.7));
     p_dest += map.GetMapCenter();
 
-    AGeoPoint loc_start(p_start, RoughAltitude(map.GetHeight(p_start) + 100));
-    AGeoPoint loc_end(p_dest, RoughAltitude(map.GetHeight(p_dest) + 100));
+    AGeoPoint loc_start(p_start, map.GetHeight(p_start).GetValueOr0() + 100);
+    AGeoPoint loc_end(p_dest, map.GetHeight(p_dest).GetValueOr0() + 100);
 
     AircraftState state;
-    GlidePolar glide_polar(fixed(0.1));
+    GlidePolar glide_polar(0.1);
     const AirspaceAircraftPerformance perf(glide_polar);
 
     GeoVector vec(loc_start, loc_end);
-    fixed range = fixed(10000) + vec.distance / 2;
+    auto range = 10000 + vec.distance / 2;
 
     state.location = loc_start;
     state.altitude = loc_start.altitude;
@@ -102,12 +106,14 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
       // dummy
 
       // real one, see if items changed
-      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), range);
+      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), range,
+                                  AirspacePredicateTrue);
       int size_1 = as_route.GetSize();
       if (verbose)
         printf("# route airspace size %d\n", size_1);
 
-      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), fixed(1));
+      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), 1,
+                                  AirspacePredicateTrue);
       int size_2 = as_route.GetSize();
       if (verbose)
         printf("# route airspace size %d\n", size_2);
@@ -115,7 +121,8 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
       ok(size_2 < size_1, "shrink as", 0);
 
       // go back
-      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_end), range);
+      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_end), range,
+                                  AirspacePredicateTrue);
       int size_3 = as_route.GetSize();
       if (verbose)
         printf("# route airspace size %d\n", size_3);
@@ -123,7 +130,8 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
       ok(size_3 >= size_2, "grow as", 0);
 
       // and again
-      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), range);
+      as_route.SynchroniseInRange(airspaces, vec.MidPoint(loc_start), range,
+                                  AirspacePredicateTrue);
       int size_4 = as_route.GetSize();
       if (verbose)
         printf("# route airspace size %d\n", size_4);
@@ -134,23 +142,24 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
     }
 
     // try the solver
-    SpeedVector wind(Angle::Degrees(0), fixed(0));
-    GlidePolar polar(fixed(1));
+    SpeedVector wind(Angle::Degrees(0), 0);
+    GlidePolar polar(1);
 
     GlideSettings settings;
     settings.SetDefaults();
-    AirspaceRoute route;
-    route.UpdatePolar(settings, polar, polar, wind);
-    route.SetTerrain(&map);
     RoutePlannerConfig config;
     config.mode = RoutePlannerConfig::Mode::BOTH;
 
-    AirspacePredicateTrue predicate;
+    AirspaceRoute route;
+    route.UpdatePolar(settings, config, polar, polar, wind);
+    route.SetTerrain(&map);
+
+    auto predicate = AirspacePredicateTrue;
 
     bool sol = false;
     for (int i = 0; i < NUM_SOL; i++) {
       loc_end.latitude += Angle::Degrees(0.1);
-      loc_end.altitude = map.GetHeight(loc_end) + 100;
+      loc_end.altitude = map.GetHeight(loc_end).GetValueOr0() + 100;
       route.Synchronise(airspaces, predicate, loc_start, loc_end);
       if (route.Solve(loc_start, loc_end, config)) {
         sol = true;
@@ -175,27 +184,32 @@ test_route(const unsigned n_airspaces, const RasterMap& map)
 int
 main(int argc, char** argv)
 {
-  const char hc_path[] = "tmp/terrain";
+  static const char map_path[] = "tmp/map.xcm";
 
-  TCHAR jp2_path[4096];
-  _tcscpy(jp2_path, PathName(hc_path));
-  _tcscat(jp2_path, _T(DIR_SEPARATOR_S) _T("terrain.jp2"));
-
-  TCHAR j2w_path[4096];
-  _tcscpy(j2w_path, PathName(hc_path));
-  _tcscat(j2w_path, _T(DIR_SEPARATOR_S) _T("terrain.j2w"));
-
-  RasterMap map(jp2_path);
-
-  NullOperationEnvironment operation;
-  if (!map.Load(jp2_path, j2w_path, operation)) {
-    fprintf(stderr, "failed to load map\n");
+  ZZIP_DIR *dir = zzip_dir_open(map_path, nullptr);
+  if (dir == nullptr) {
+    fprintf(stderr, "Failed to open %s\n", map_path);
     return EXIT_FAILURE;
   }
 
+  RasterMap map;
+
+  NullOperationEnvironment operation;
+  if (!LoadTerrainOverview(dir, map.GetTileCache(), operation)) {
+    fprintf(stderr, "failed to load map\n");
+    zzip_dir_close(dir);
+    return EXIT_FAILURE;
+  }
+
+  map.UpdateProjection();
+
+  SharedMutex mutex;
   do {
-    map.SetViewCenter(map.GetMapCenter(), fixed(100000));
+    UpdateTerrainTiles(dir, map.GetTileCache(), mutex,
+                       map.GetProjection(),
+                       map.GetMapCenter(), 100000);
   } while (map.IsDirty());
+  zzip_dir_close(dir);
 
   plan_tests(4 + NUM_SOL);
   ok(test_route(28, map), "route 28", 0);
